@@ -3,6 +3,7 @@
 #include "proxysql.h"
 #include "cpp.h"
 
+#define PROXYSQL_LOGGER_PTHREAD_MUTEX
 
 class MySQL_Event {
 	private:
@@ -20,7 +21,7 @@ class MySQL_Event {
 	char *client;
 	size_t server_len;
 	size_t client_len;
-	uint64_t total_length;
+	//uint64_t total_length;
 	unsigned char buf[10];
 	enum log_event_type et;
 	uint64_t hid;
@@ -39,7 +40,11 @@ class MySQL_Logger {
 	char *datadir;
 	unsigned int log_file_id;
 	unsigned int max_log_file_size;
+#ifdef PROXYSQL_LOGGER_PTHREAD_MUTEX
+	pthread_mutex_t wmutex;
+#else
 	rwlock_t rwlock;
+#endif
 	void wrlock();
 	void wrunlock();
 	std::fstream *logfile;
