@@ -6,6 +6,7 @@
 #ifdef IDLE_THREADS
 #include <sys/epoll.h>
 #endif // IDLE_THREADS
+#include <atomic>
 
 #define MIN_POLL_LEN 8
 #define MIN_POLL_DELETE_RATIO  8
@@ -186,6 +187,7 @@ class MySQL_Thread
 	unsigned long long curtime;
 	unsigned long long pre_poll_time;
 	unsigned long long last_maintenance_time;
+	std::atomic<unsigned long long> atomic_curtime;
 	PtrArray *mysql_sessions;
 	PtrArray *mirror_queue_mysql_sessions;
 	PtrArray *mirror_queue_mysql_sessions_cache;
@@ -227,6 +229,10 @@ class MySQL_Thread
 		unsigned int active_transactions;
 	} status_variables;
 
+	struct {
+		bool stats_time_backend_query;
+		bool stats_time_query_processor;
+	} variables;
 
 #ifdef PROXYSQL_MYSQL_PTHREAD_MUTEX
   pthread_mutex_t thread_mutex;
@@ -327,6 +333,7 @@ class MySQL_Threads_Handler
 		int monitor_slave_lag_when_null;
 		char *monitor_username;
 		char *monitor_password;
+		char * monitor_replication_lag_use_percona_heartbeat;
 		int ping_interval_server_msec;
 		int ping_timeout_server;
 		int shun_on_failures;
@@ -360,13 +367,18 @@ class MySQL_Threads_Handler
 		bool forward_autocommit;
 		bool enforce_autocommit_on_reads;
 		bool autocommit_false_not_reusable;
+		bool autocommit_false_is_transaction;
+		bool verbose_query_error;
 		int max_allowed_packet;
+		int throttle_connections_per_sec_to_hostgroup;
 		int max_transaction_time;
 		int threshold_query_length;
 		int threshold_resultset_size;
 		int query_digests_max_digest_length;
 		int query_digests_max_query_length;
 		int wait_timeout;
+		int throttle_max_bytes_per_second_to_client;
+		int throttle_ratio_server_to_client;
 		int max_connections;
 		int max_stmts_per_connection;
 		int max_stmts_cache;
@@ -388,6 +400,7 @@ class MySQL_Threads_Handler
 		uint16_t server_capabilities;
 		int poll_timeout;
 		int poll_timeout_on_failure;
+		int connpoll_reset_queue_length;
 		char *eventslog_filename;
 		int eventslog_filesize;
 		// SSL related, proxy to server
@@ -396,6 +409,8 @@ class MySQL_Threads_Handler
 		char * ssl_p2s_key;
 		char * ssl_p2s_cipher;
 		int query_cache_size_MB;
+		bool stats_time_backend_query;
+		bool stats_time_query_processor;
 	} variables;
 	struct {
 		unsigned int mirror_sessions_current;
@@ -433,7 +448,7 @@ class MySQL_Threads_Handler
 	void stop_listeners();
 	void signal_all_threads(unsigned char _c=0);
 	SQLite3_result * SQL3_Processlist();
-	SQLite3_result * SQL3_GlobalStatus();
+	SQLite3_result * SQL3_GlobalStatus(bool _memory);
 	bool kill_session(uint32_t _thread_session_id);
 	unsigned long long get_total_mirror_queue();
 	unsigned long long get_total_backend_stmt_prepare();
