@@ -291,6 +291,7 @@ enum MYSQL_COM_QUERY_command {
 	MYSQL_COM_QUERY_USE,
 	MYSQL_COM_QUERY_SHOW,
 	MYSQL_COM_QUERY_UNKNOWN,
+	MYSQL_COM_QUERY__UNINITIALIZED,
 	MYSQL_COM_QUERY___NONE // Special marker.
 };
 
@@ -464,6 +465,7 @@ struct _global_variables_t {
 	pthread_rwlock_t rwlock_usernames;
 
 	bool has_debug;
+	bool idle_threads;
 
 	volatile int shutdown;
 	bool nostart;
@@ -588,10 +590,13 @@ __thread char *mysql_thread___init_connect;
 __thread char *mysql_thread___default_sql_mode;
 __thread char *mysql_thread___default_time_zone;
 __thread int mysql_thread___max_allowed_packet;
+__thread int mysql_thread___throttle_connections_per_sec_to_hostgroup;
 __thread int mysql_thread___max_transaction_time;
 __thread int mysql_thread___threshold_query_length;
 __thread int mysql_thread___threshold_resultset_size;
 __thread int mysql_thread___wait_timeout;
+__thread int mysql_thread___throttle_max_bytes_per_second_to_client;
+__thread int mysql_thread___throttle_ratio_server_to_client;
 __thread int mysql_thread___max_connections;
 __thread int mysql_thread___max_stmts_per_connection;
 __thread int mysql_thread___max_stmts_cache;
@@ -625,6 +630,8 @@ __thread bool mysql_thread___multiplexing;
 __thread bool mysql_thread___forward_autocommit;
 __thread bool mysql_thread___enforce_autocommit_on_reads;
 __thread bool mysql_thread___autocommit_false_not_reusable;
+__thread bool mysql_thread___autocommit_false_is_transaction;
+__thread bool mysql_thread___verbose_query_error;
 __thread bool mysql_thread___servers_stats;
 __thread bool mysql_thread___commands_stats;
 __thread bool mysql_thread___query_digests;
@@ -670,6 +677,7 @@ __thread int mysql_thread___monitor_query_timeout;
 __thread int mysql_thread___monitor_slave_lag_when_null;
 __thread char * mysql_thread___monitor_username;
 __thread char * mysql_thread___monitor_password;
+__thread char * mysql_thread___monitor_replication_lag_use_percona_heartbeat;
 
 #ifdef DEBUG
 __thread bool mysql_thread___session_debug;
@@ -687,10 +695,13 @@ extern __thread char *mysql_thread___init_connect;
 extern __thread char *mysql_thread___default_sql_mode;
 extern __thread char *mysql_thread___default_time_zone;
 extern __thread int mysql_thread___max_allowed_packet;
+extern __thread int mysql_thread___throttle_connections_per_sec_to_hostgroup;
 extern __thread int mysql_thread___max_transaction_time;
 extern __thread int mysql_thread___threshold_query_length;
 extern __thread int mysql_thread___threshold_resultset_size;
 extern __thread int mysql_thread___wait_timeout;
+extern __thread int mysql_thread___throttle_max_bytes_per_second_to_client;
+extern __thread int mysql_thread___throttle_ratio_server_to_client;
 extern __thread int mysql_thread___max_connections;
 extern __thread int mysql_thread___max_stmts_per_connection;
 extern __thread int mysql_thread___max_stmts_cache;
@@ -724,6 +735,8 @@ extern __thread bool mysql_thread___multiplexing;
 extern __thread bool mysql_thread___forward_autocommit;
 extern __thread bool mysql_thread___enforce_autocommit_on_reads;
 extern __thread bool mysql_thread___autocommit_false_not_reusable;
+extern __thread bool mysql_thread___autocommit_false_is_transaction;
+extern __thread bool mysql_thread___verbose_query_error;
 extern __thread bool mysql_thread___servers_stats;
 extern __thread bool mysql_thread___commands_stats;
 extern __thread bool mysql_thread___query_digests;
@@ -769,6 +782,7 @@ extern __thread int mysql_thread___monitor_query_timeout;
 extern __thread int mysql_thread___monitor_slave_lag_when_null;
 extern __thread char * mysql_thread___monitor_username;
 extern __thread char * mysql_thread___monitor_password;
+extern __thread char * mysql_thread___monitor_replication_lag_use_percona_heartbeat;
 
 #ifdef DEBUG
 extern __thread bool mysql_thread___session_debug;
