@@ -3,14 +3,13 @@
 
 #include "SpookyV2.h"
 
-#ifndef PROXYSQL_STMT_V14
-extern MySQL_STMT_Manager *GloMyStmt;
-static uint32_t add_prepared_statement_calls = 0;
-static uint32_t find_prepared_statement_by_hash_calls = 0;
-#else
+//#ifndef PROXYSQL_STMT_V14
+//extern MySQL_STMT_Manager *GloMyStmt;
+//static uint32_t add_prepared_statement_calls = 0;
+//static uint32_t find_prepared_statement_by_hash_calls = 0;
+//#else
 extern MySQL_STMT_Manager_v14 *GloMyStmt;
-#endif
-
+//#endif
 
 static uint64_t stmt_compute_hash(unsigned int hostgroup, char *user,
                                   char *schema, char *query,
@@ -94,6 +93,7 @@ bool StmtLongDataHandler::add(uint32_t _stmt_id, uint16_t _param_id,
 	sld->param_id = _param_id;
 	sld->size = _size;
 	sld->data = malloc(_size);
+	sld->is_null = 0; // because the client is sending data, the field cannot be NULL
 	memcpy(sld->data, _data, _size);
 	long_datas->add(sld);
 	return false;  // a new entry was created
@@ -118,7 +118,7 @@ unsigned int StmtLongDataHandler::reset(uint32_t _stmt_id) {
 }
 
 void *StmtLongDataHandler::get(uint32_t _stmt_id, uint16_t _param_id,
-                               unsigned long **_size) {
+                               unsigned long **_size, my_bool **_is_null) {
 	stmt_long_data_t *sld = NULL;
 	unsigned int i;
 	for (i = 0; i < long_datas->len; i++) {
@@ -126,17 +126,18 @@ void *StmtLongDataHandler::get(uint32_t _stmt_id, uint16_t _param_id,
 		if (sld->stmt_id == _stmt_id && sld->param_id == _param_id) {
 			// we found it!
 			*_size = &sld->size;
+			*_is_null = &sld->is_null;
 			return sld->data;
 		}
 	}
 	return NULL;
 }
 
-#ifndef PROXYSQL_STMT_V14
-MySQL_STMT_Global_info::MySQL_STMT_Global_info(uint32_t id, unsigned int h,
-#else
+//#ifndef PROXYSQL_STMT_V14
+//MySQL_STMT_Global_info::MySQL_STMT_Global_info(uint32_t id, unsigned int h,
+//#else
 MySQL_STMT_Global_info::MySQL_STMT_Global_info(uint64_t id, unsigned int h,
-#endif
+//#endif
                                                char *u, char *s, char *q,
                                                unsigned int ql,
                                                MYSQL_STMT *stmt, uint64_t _h) {
@@ -222,6 +223,192 @@ MySQL_STMT_Global_info::MySQL_STMT_Global_info(uint64_t id, unsigned int h,
 	}
 }
 
+#ifdef PROXYSQL_STMT_V14
+void MySQL_STMT_Global_info::update_metadata(MYSQL_STMT *stmt) {
+	int i;
+	bool need_refresh = false;
+	if (
+		(num_params != stmt->param_count)
+		||
+		(num_columns != stmt->field_count)
+	) {
+		need_refresh = true;
+	}
+	for (i = 0; i < num_columns; i++) {
+		if (need_refresh == false) { // don't bother to check if need_refresh == true
+			bool ok = true;
+			MYSQL_FIELD *fs = &(stmt->fields[i]);
+			MYSQL_FIELD *fd = fields[i];
+			if (ok) {
+				ok = false;
+				if (fd->name == NULL && fs->name == NULL) {
+					ok = true;
+				} else {
+					if (fd->name && fs->name && strcmp(fd->name,fs->name)==0) {
+						ok = true;
+					}
+				}
+			}
+			if (ok) {
+				ok = false;
+				if (fd->org_name == NULL && fs->org_name == NULL) {
+					ok = true;
+				} else {
+					if (fd->org_name && fs->org_name && strcmp(fd->org_name,fs->org_name)==0) {
+						ok = true;
+					}
+				}
+			}
+			if (ok) {
+				ok = false;
+				if (fd->table == NULL && fs->table == NULL) {
+					ok = true;
+				} else {
+					if (fd->table && fs->table && strcmp(fd->table,fs->table)==0) {
+						ok = true;
+					}
+				}
+			}
+			if (ok) {
+				ok = false;
+				if (fd->org_table == NULL && fs->org_table == NULL) {
+					ok = true;
+				} else {
+					if (fd->org_table && fs->org_table && strcmp(fd->org_table,fs->org_table)==0) {
+						ok = true;
+					}
+				}
+			}
+			if (ok) {
+				ok = false;
+				if (fd->db == NULL && fs->db == NULL) {
+					ok = true;
+				} else {
+					if (fd->db && fs->db && strcmp(fd->db,fs->db)==0) {
+						ok = true;
+					}
+				}
+			}
+			if (ok) {
+				ok = false;
+				if (fd->catalog == NULL && fs->catalog == NULL) {
+					ok = true;
+				} else {
+					if (fd->catalog && fs->catalog && strcmp(fd->catalog,fs->catalog)==0) {
+						ok = true;
+					}
+				}
+			}
+			if (ok) {
+				ok = false;
+				if (fd->def == NULL && fs->def == NULL) {
+					ok = true;
+				} else {
+					if (fd->def && fs->def && strcmp(fd->def,fs->def)==0) {
+						ok = true;
+					}
+				}
+			}
+			if (ok == false) {
+				need_refresh = true;
+			}
+		}
+	}
+	if (need_refresh) {
+		proxy_warning("Updating metadata for stmt %lu , user %s, query %s\n", statement_id, username, query);
+// from here is copied from destructor
+		if (num_columns) {
+			uint16_t i;
+			for (i = 0; i < num_columns; i++) {
+				MYSQL_FIELD *f = fields[i];
+				if (f->name) {
+					free(f->name);
+					f->name = NULL;
+				}
+				if (f->org_name) {
+					free(f->org_name);
+					f->org_name = NULL;
+				}
+				if (f->table) {
+					free(f->table);
+					f->table = NULL;
+				}
+				if (f->org_table) {
+					free(f->org_table);
+					f->org_table = NULL;
+				}
+				if (f->db) {
+					free(f->db);
+					f->db = NULL;
+				}
+				if (f->catalog) {
+					free(f->catalog);
+					f->catalog = NULL;
+				}
+				if (f->def) {
+					free(f->def);
+					f->def = NULL;
+				}
+				free(fields[i]);
+			}
+			free(fields);
+			fields = NULL;
+		}
+		if (num_params) {
+			uint16_t i;
+			for (i = 0; i < num_params; i++) {
+				free(params[i]);
+			}
+			free(params);
+			params = NULL;
+		}
+// till here is copied from destructor
+
+// from here is copied from constructor
+		num_params = stmt->param_count;
+		num_columns = stmt->field_count;
+		fields = NULL;
+		if (num_columns) {
+			fields = (MYSQL_FIELD **)malloc(num_columns * sizeof(MYSQL_FIELD *));
+			uint16_t i;
+			for (i = 0; i < num_columns; i++) {
+				fields[i] = (MYSQL_FIELD *)malloc(sizeof(MYSQL_FIELD));
+				MYSQL_FIELD *fs = &(stmt->fields[i]);
+				MYSQL_FIELD *fd = fields[i];
+				// first copy all fields
+				memcpy(fd, fs, sizeof(MYSQL_FIELD));
+				// then duplicate strings
+				fd->name = (fs->name ? strdup(fs->name) : NULL);
+				fd->org_name = (fs->org_name ? strdup(fs->org_name) : NULL);
+				fd->table = (fs->table ? strdup(fs->table) : NULL);
+				fd->org_table = (fs->org_table ? strdup(fs->org_table) : NULL);
+				fd->db = (fs->db ? strdup(fs->db) : NULL);
+				fd->catalog = (fs->catalog ? strdup(fs->catalog) : NULL);
+				fd->def = (fs->def ? strdup(fs->def) : NULL);
+			}
+		}
+	
+		params = NULL;
+		if (num_params == 2) {
+			PROXY_TRACE();
+		}
+		if (num_params) {
+			params = (MYSQL_BIND **)malloc(num_params * sizeof(MYSQL_BIND *));
+			uint16_t i;
+			for (i = 0; i < num_params; i++) {
+				params[i] = (MYSQL_BIND *)malloc(sizeof(MYSQL_BIND));
+				// MYSQL_BIND *ps=&(stmt->params[i]);
+				// MYSQL_BIND *pd=params[i];
+				// copy all params
+				// memcpy(pd,ps,sizeof(MYSQL_BIND));
+				memset(params[i], 0, sizeof(MYSQL_BIND));
+			}
+		}
+// till here is copied from constructor
+	}
+}
+#endif
+
 MySQL_STMT_Global_info::~MySQL_STMT_Global_info() {
 	free(username);
 	free(schemaname);
@@ -277,6 +464,7 @@ MySQL_STMT_Global_info::~MySQL_STMT_Global_info() {
 		digest_text = NULL;
 	}
 }
+/*
 #ifndef PROXYSQL_STMT_V14
 uint64_t MySQL_STMTs_local::compute_hash(unsigned int hostgroup, char *user,
                                          char *schema, char *query,
@@ -569,6 +757,7 @@ MySQL_STMT_Global_info *MySQL_STMT_Manager::find_prepared_statement_by_hash(
 
 
 #else // PROXYSQL_STMT_V14
+*/
 extern MySQL_STMT_Manager_v14 *GloMyStmt;
 
 void MySQL_STMTs_local_v14::backend_insert(uint64_t global_statement_id, MYSQL_STMT *stmt) {
@@ -591,6 +780,10 @@ uint64_t MySQL_STMTs_local_v14::compute_hash(unsigned int hostgroup, char *user,
 
 MySQL_STMT_Manager_v14::MySQL_STMT_Manager_v14() {
 	pthread_rwlock_init(&rwlock_, NULL);
+        map_stmt_id_to_info= std::map<uint64_t, MySQL_STMT_Global_info *>();       // map using statement id
+        map_stmt_hash_to_info = std::map<uint64_t, MySQL_STMT_Global_info *>();     // map using hashes
+        free_stmt_ids = std::stack<uint64_t> ();
+
 	next_statement_id =
 	    1;  // we initialize this as 1 because we 0 is not allowed
 }
@@ -605,6 +798,41 @@ void MySQL_STMT_Manager_v14::ref_count_client(uint64_t _stmt_id ,int _v, bool lo
 	if (s != map_stmt_id_to_info.end()) {
 		MySQL_STMT_Global_info *stmt_info = s->second;
 		stmt_info->ref_count_client += _v;
+			if (map_stmt_id_to_info.size() > (unsigned)mysql_thread___max_stmts_cache) {
+				int max_purge = map_stmt_id_to_info.size() / 20;  // purge up to 5%
+				int i = -1;
+				uint64_t *torem =
+				    (uint64_t *)malloc(max_purge * sizeof(uint64_t));
+				for (std::map<uint64_t, MySQL_STMT_Global_info *>::iterator it =
+				         map_stmt_id_to_info.begin();
+				     it != map_stmt_id_to_info.end(); ++it) {
+					if (i == (max_purge - 1)) continue;
+					MySQL_STMT_Global_info *a = it->second;
+					if (__sync_add_and_fetch(&a->ref_count_client, 0) == 0) {
+						uint64_t hash = a->hash;
+						auto s2 = map_stmt_hash_to_info.find(hash);
+						if (s2 != map_stmt_hash_to_info.end()) {
+							map_stmt_hash_to_info.erase(s2);
+						}
+						// m.erase(it);
+						// delete a;
+						i++;
+						torem[i] = it->first;
+					}
+				}
+				while (i >= 0) {
+					uint64_t id = torem[i];
+					auto s3 = map_stmt_id_to_info.find(id);
+					MySQL_STMT_Global_info *a = s3->second;
+					if (a->ref_count_server == 0) {
+						free_stmt_ids.push(id);
+					}
+					map_stmt_id_to_info.erase(s3);
+					delete a;
+					i--;
+				}
+				free(torem);
+			}
 	}
 	if (lock)
 		pthread_rwlock_unlock(&rwlock_);
@@ -613,7 +841,8 @@ void MySQL_STMT_Manager_v14::ref_count_client(uint64_t _stmt_id ,int _v, bool lo
 void MySQL_STMT_Manager_v14::ref_count_server(uint64_t _stmt_id ,int _v, bool lock) {
 	if (lock)
 		pthread_rwlock_wrlock(&rwlock_);
-	auto s = map_stmt_id_to_info.find(_stmt_id);
+	std::map<uint64_t, MySQL_STMT_Global_info *>::iterator s;
+	s = map_stmt_id_to_info.find(_stmt_id);
 	if (s != map_stmt_id_to_info.end()) {
 		MySQL_STMT_Global_info *stmt_info = s->second;
 		stmt_info->ref_count_server += _v;
@@ -712,6 +941,15 @@ MySQL_STMT_Global_info *MySQL_STMT_Manager_v14::find_prepared_statement_by_stmt_
 
 uint32_t MySQL_STMTs_local_v14::generate_new_client_stmt_id(uint64_t global_statement_id) {
 	uint32_t ret=0;
+/*
+	//auto s2 = global_stmt_to_client_ids.find(global_statement_id);
+	std::pair<std::multimap<uint64_t,uint32_t>::iterator, std::multimap<uint64_t,uint32_t>::iterator> itret;
+	itret = global_stmt_to_client_ids.equal_range(global_statement_id);
+	for (std::multimap<uint64_t,uint32_t>::iterator it=itret.first; it!=itret.second; ++it) {
+		ret = it->second;
+		return ret;
+	}
+*/
 	if (free_client_ids.size()) {
 		ret=free_client_ids.top();
 		free_client_ids.pop();
@@ -772,27 +1010,28 @@ MySQL_STMT_Global_info *MySQL_STMT_Manager_v14::add_prepared_statement(
 		// MySQL_STMT_Global_info *a=f->second;
 		// ret=a->statement_id;
 		ret = f->second;
+		ret->update_metadata(stmt);
 		//*is_new = false;
 	} else {
 		// FIXME: add a stack here too!!!
 		// we need to create a new one
-/*
+
 		bool free_id_avail = false;
 		free_id_avail = free_stmt_ids.size();
 
-		uint32_t next_id = 0;
+		uint64_t next_id = 0;
 		if (free_id_avail) {
 			next_id = free_stmt_ids.top();
 			free_stmt_ids.pop();
 		} else {
-			// next_id = next_statement_id;
-			// next_statement_id++;
-			__sync_fetch_and_add(&next_statement_id, 1);
+			next_id = next_statement_id;
+			next_statement_id++;
+			//__sync_fetch_and_add(&next_statement_id, 1);
 		}
-*/
-		next_statement_id++;
+
+		//next_statement_id++;
 		MySQL_STMT_Global_info *a =
-		    new MySQL_STMT_Global_info(next_statement_id, _h, u, s, q, ql, stmt, hash);
+		    new MySQL_STMT_Global_info(next_id, _h, u, s, q, ql, stmt, hash);
 		a->properties.cache_ttl = _cache_ttl;
 		a->properties.timeout = _timeout;
 		a->properties.delay = _delay;
@@ -853,4 +1092,110 @@ void MySQL_STMT_Manager_v14::get_metrics(uint64_t *c_unique, uint64_t *c_total,
 	*s_total = s_t;
 }
 
-#endif // PROXYSQL_STMT_V14
+//#endif // PROXYSQL_STMT_V14
+
+
+class PS_global_stats {
+	public:
+	uint64_t statement_id;
+	unsigned int hid;
+	char *username;
+	char *schemaname;
+	uint64_t digest;
+	unsigned long long ref_count_client;
+	unsigned long long ref_count_server;
+	char *query;
+	PS_global_stats(uint64_t stmt_id, unsigned int h, char *u, char *s, uint64_t d, char *q, unsigned long long ref_c, unsigned long long ref_s) {
+		statement_id = stmt_id;
+		hid=h;
+		digest=d;
+		query=strndup(q, mysql_thread___query_digests_max_digest_length);
+		username=strdup(u);
+		schemaname=strdup(s);
+		ref_count_client = ref_c;
+		ref_count_server = ref_s;
+	}
+	~PS_global_stats() {
+		if (query) {
+			free(query);
+			query=NULL;
+		}
+		if (username) {
+			free(username);
+			username=NULL;
+		}
+		if (schemaname) {
+			free(schemaname);
+			schemaname=NULL;
+		}
+	}
+	char **get_row() {
+		char buf[128];
+		char **pta=(char **)malloc(sizeof(char *)*8);
+		sprintf(buf,"%llu",statement_id);
+		pta[0]=strdup(buf);
+		sprintf(buf,"%u",hid);
+		pta[1]=strdup(buf);
+		assert(schemaname);
+		pta[2]=strdup(schemaname);
+		assert(username);
+		pta[3]=strdup(username);
+
+		sprintf(buf,"0x%016llX", (long long unsigned int)digest);
+		pta[4]=strdup(buf);
+
+		assert(query);
+		pta[5]=strdup(query);
+		sprintf(buf,"%llu",ref_count_client);
+		pta[6]=strdup(buf);
+		sprintf(buf,"%llu",ref_count_server);
+		pta[7]=strdup(buf);
+
+		return pta;
+	}
+	void free_row(char **pta) {
+		int i;
+		for (i=0;i<8;i++) {
+			assert(pta[i]);
+			free(pta[i]);
+		}
+		free(pta);
+	}
+};
+
+
+SQLite3_result * MySQL_STMT_Manager_v14::get_prepared_statements_global_infos() {
+	proxy_debug(PROXY_DEBUG_MYSQL_QUERY_PROCESSOR, 4, "Dumping current prepared statements global info\n");
+	SQLite3_result *result=new SQLite3_result(8);
+	rdlock();
+	result->add_column_definition(SQLITE_TEXT,"stmt_id");
+	result->add_column_definition(SQLITE_TEXT,"hid");
+	result->add_column_definition(SQLITE_TEXT,"schemaname");
+	result->add_column_definition(SQLITE_TEXT,"username");
+	result->add_column_definition(SQLITE_TEXT,"digest");
+	result->add_column_definition(SQLITE_TEXT,"query");
+	result->add_column_definition(SQLITE_TEXT,"ref_count_client");
+	result->add_column_definition(SQLITE_TEXT,"ref_count_server");
+	for (std::map<uint64_t, MySQL_STMT_Global_info *>::iterator it = map_stmt_id_to_info.begin();
+			it != map_stmt_id_to_info.end(); ++it) {
+		MySQL_STMT_Global_info *a = it->second;
+		PS_global_stats * pgs = new PS_global_stats(a->statement_id, a->hostgroup_id,
+			a->schemaname, a->username,
+			a->hash, a->query,
+			a->ref_count_client, a->ref_count_server);
+			char **pta = pgs->get_row();
+			result->add_row(pta);
+			pgs->free_row(pta);
+			delete pgs;
+	}
+/*
+	for (std::unordered_map<uint64_t, void *>::iterator it=digest_umap.begin(); it!=digest_umap.end(); ++it) {
+		QP_query_digest_stats *qds=(QP_query_digest_stats *)it->second;
+		char **pta=qds->get_row();
+		result->add_row(pta);
+		qds->free_row(pta);
+	}
+*/
+	unlock();
+	return result;
+}
