@@ -451,15 +451,21 @@ bool MySQL_Protocol::generate_statistics_response(bool send, void **ptr, unsigne
 	return true;
 }
 
-bool MySQL_Protocol::generate_pkt_EOF(bool send, void **ptr, unsigned int *len, uint8_t sequence_id, uint16_t warnings, uint16_t status) {
+bool MySQL_Protocol::generate_pkt_EOF(bool send, void **ptr, unsigned int *len, uint8_t sequence_id, uint16_t warnings, uint16_t status, MySQL_ResultSet *myrs) {
 	if ((*myds)->sess->mirror==true) {
 		return true;
 	}
 	mysql_hdr myhdr;
 	myhdr.pkt_id=sequence_id;
 	myhdr.pkt_length=5;
-  unsigned int size=myhdr.pkt_length+sizeof(mysql_hdr);
-  unsigned char *_ptr=(unsigned char *)l_alloc(size);
+	unsigned int size=myhdr.pkt_length+sizeof(mysql_hdr);
+	unsigned char *_ptr = NULL;
+	if (myrs == NULL) {
+		_ptr = (unsigned char *)l_alloc(size);
+	} else {
+		_ptr = myrs->buffer + myrs->buffer_used;
+		myrs->buffer_used += size;
+	}
   memcpy(_ptr, &myhdr, sizeof(mysql_hdr));
   int l=sizeof(mysql_hdr);
 	_ptr[l]=0xfe; l++;
@@ -583,7 +589,7 @@ bool MySQL_Protocol::generate_pkt_OK(bool send, void **ptr, unsigned int *len, u
 	return true;
 }
 
-bool MySQL_Protocol::generate_pkt_column_count(bool send, void **ptr, unsigned int *len, uint8_t sequence_id, uint64_t count) {
+bool MySQL_Protocol::generate_pkt_column_count(bool send, void **ptr, unsigned int *len, uint8_t sequence_id, uint64_t count, MySQL_ResultSet *myrs) {
 	if ((*myds)->sess->mirror==true) {
 		return true;
 	}
@@ -595,7 +601,29 @@ bool MySQL_Protocol::generate_pkt_column_count(bool send, void **ptr, unsigned i
 	myhdr.pkt_id=sequence_id;
 	myhdr.pkt_length=count_len;
   unsigned int size=myhdr.pkt_length+sizeof(mysql_hdr);
-  unsigned char *_ptr=(unsigned char *)l_alloc(size);
+//  unsigned char *_ptr=(unsigned char *)l_alloc(size);
+  unsigned char *_ptr = NULL;
+	if (myrs) {
+		if ( size<=(RESULTSET_BUFLEN-myrs->buffer_used) ) {
+			// there is space in the buffer, add the data to it
+			_ptr = myrs->buffer + myrs->buffer_used;
+			myrs->buffer_used += size;
+		} else {
+			// there is no space in the buffer, we flush the buffer and recreate it
+			myrs->buffer_to_PSarrayOut();
+			// now we can check again if there is space in the buffer
+			if ( size<=(RESULTSET_BUFLEN-myrs->buffer_used) ) {
+				// there is space in the NEW buffer, add the data to it
+				_ptr = myrs->buffer + myrs->buffer_used;
+				myrs->buffer_used += size;
+			} else {
+				// a new buffer is not enough to store the new row
+				_ptr=(unsigned char *)l_alloc(size);
+			}
+		}
+	} else {
+		_ptr=(unsigned char *)l_alloc(size);
+	}
   memcpy(_ptr, &myhdr, sizeof(mysql_hdr));
   int l=sizeof(mysql_hdr);
 
@@ -607,10 +635,18 @@ bool MySQL_Protocol::generate_pkt_column_count(bool send, void **ptr, unsigned i
 #ifdef DEBUG
 	if (dump_pkt) { __dump_pkt(__func__,_ptr,size); }
 #endif
+	if (myrs) {
+		if (_ptr >= myrs->buffer && _ptr < myrs->buffer+RESULTSET_BUFLEN) {
+			// we are writing within the buffer, do not add to PSarrayOUT
+		} else {
+			// we are writing outside the buffer, add to PSarrayOUT
+			myrs->PSarrayOUT.add(_ptr,size);
+		}
+	}
 	return true;
 }
 
-bool MySQL_Protocol::generate_pkt_field(bool send, void **ptr, unsigned int *len, uint8_t sequence_id, char *schema, char *table, char *org_table, char *name, char *org_name, uint16_t charset, uint32_t column_length, uint8_t type, uint16_t flags, uint8_t decimals, bool field_list, uint64_t defvalue_length, char *defvalue) {
+bool MySQL_Protocol::generate_pkt_field(bool send, void **ptr, unsigned int *len, uint8_t sequence_id, char *schema, char *table, char *org_table, char *name, char *org_name, uint16_t charset, uint32_t column_length, uint8_t type, uint16_t flags, uint8_t decimals, bool field_list, uint64_t defvalue_length, char *defvalue, MySQL_ResultSet *myrs) {
 
 	if ((*myds)->sess->mirror==true) {
 		return true;
@@ -664,7 +700,28 @@ bool MySQL_Protocol::generate_pkt_field(bool send, void **ptr, unsigned int *len
 	} //else myhdr.pkt_length++;
 
   unsigned int size=myhdr.pkt_length+sizeof(mysql_hdr);
-  unsigned char *_ptr=(unsigned char *)l_alloc(size);
+  unsigned char *_ptr = NULL;
+	if (myrs) {
+		if ( size<=(RESULTSET_BUFLEN-myrs->buffer_used) ) {
+			// there is space in the buffer, add the data to it
+			_ptr = myrs->buffer + myrs->buffer_used;
+			myrs->buffer_used += size;
+		} else {
+			// there is no space in the buffer, we flush the buffer and recreate it
+			myrs->buffer_to_PSarrayOut();
+			// now we can check again if there is space in the buffer
+			if ( size<=(RESULTSET_BUFLEN-myrs->buffer_used) ) {
+				// there is space in the NEW buffer, add the data to it
+				_ptr = myrs->buffer + myrs->buffer_used;
+				myrs->buffer_used += size;
+			} else {
+				// a new buffer is not enough to store the new row
+				_ptr=(unsigned char *)l_alloc(size);
+			}
+		}
+	} else {
+		_ptr=(unsigned char *)l_alloc(size);
+	}
   memcpy(_ptr, &myhdr, sizeof(mysql_hdr));
   int l=sizeof(mysql_hdr);
 
@@ -678,7 +735,9 @@ bool MySQL_Protocol::generate_pkt_field(bool send, void **ptr, unsigned int *len
 	memcpy(_ptr+l,&charset,sizeof(uint16_t)); l+=sizeof(uint16_t);
 	memcpy(_ptr+l,&column_length,sizeof(uint32_t)); l+=sizeof(uint32_t);
 	_ptr[l]=type; l++;
-	memcpy(_ptr+l,&flags,sizeof(uint16_t)); l+=sizeof(uint16_t);
+	uint16_t flags_ = flags;
+	flags_ = flags_ & ~NUM_FLAG;
+	memcpy(_ptr+l,&flags_,sizeof(uint16_t)); l+=sizeof(uint16_t);
 	_ptr[l]=decimals; l++;
 	_ptr[l]=0x00; l++;
 	_ptr[l]=0x00; l++;
@@ -693,6 +752,14 @@ bool MySQL_Protocol::generate_pkt_field(bool send, void **ptr, unsigned int *len
 #ifdef DEBUG
 	if (dump_pkt) { __dump_pkt(__func__,_ptr,size); }
 #endif
+	if (myrs) {
+		if (_ptr >= myrs->buffer && _ptr < myrs->buffer+RESULTSET_BUFLEN) {
+			// we are writing within the buffer, do not add to PSarrayOUT
+		} else {
+			// we are writing outside the buffer, add to PSarrayOUT
+			myrs->PSarrayOUT.add(_ptr,size);
+		}
+	}
 	return true;
 }
 
@@ -720,6 +787,13 @@ bool MySQL_Protocol::generate_STMT_PREPARE_RESPONSE(uint8_t sequence_id, MySQL_S
 	memcpy(okpack+14,&stmt_info->warning_count,sizeof(uint16_t));
 	(*myds)->PSarrayOUT->add((void *)okpack,16);
 	sid++;
+	int setStatus = SERVER_STATUS_AUTOCOMMIT;
+	if (myds) {
+		setStatus = 0;
+		unsigned int Trx_id = (*myds)->sess->FindOneActiveTransaction();
+		setStatus = (Trx_id >= 0 ? SERVER_STATUS_IN_TRANS : 0 );
+		if ((*myds)->sess->autocommit) setStatus += SERVER_STATUS_AUTOCOMMIT;
+	}
 	if (stmt_info->num_params) {
 		for (i=0; i<stmt_info->num_params; i++) {
 			generate_pkt_field(true,NULL,NULL,sid,
@@ -727,7 +801,7 @@ bool MySQL_Protocol::generate_STMT_PREPARE_RESPONSE(uint8_t sequence_id, MySQL_S
 				63,0,253,128,0,false,0,NULL); // NOTE: charset is 63 = binary !
 			sid++;
 		}
-		generate_pkt_EOF(true,NULL,NULL,sid,0,SERVER_STATUS_AUTOCOMMIT); // FIXME : for now we pass a very broken flag
+		generate_pkt_EOF(true,NULL,NULL,sid,0,setStatus);
 		sid++;
 	}
 	if (stmt_info->num_columns) {
@@ -737,10 +811,10 @@ bool MySQL_Protocol::generate_STMT_PREPARE_RESPONSE(uint8_t sequence_id, MySQL_S
 				fd->db,
 				fd->table, fd->org_table,
 				fd->name, fd->org_name,
-				fd->charsetnr, 0, fd->type, fd->flags, fd->decimals, false,0,NULL);
+				fd->charsetnr, fd->length, fd->type, fd->flags, fd->decimals, false,0,NULL);
 			sid++;
 		}
-		generate_pkt_EOF(true,NULL,NULL,sid,0,SERVER_STATUS_AUTOCOMMIT); // FIXME : for now we pass a very broken flag
+		generate_pkt_EOF(true,NULL,NULL,sid,0,setStatus);
 		sid++;
 	}
 	return true;
@@ -829,7 +903,7 @@ uint8_t MySQL_Protocol::generate_pkt_row3(MySQL_ResultSet *myrs, unsigned int *l
 			// we are writing within the buffer, do not add to PSarrayOUT
 		} else {
 			// we are writing outside the buffer, add to PSarrayOUT
-			myrs->PSarrayOUT->add(pkt.ptr,pkt.size);
+			myrs->PSarrayOUT.add(pkt.ptr,pkt.size);
 		}
 	} else {
 		unsigned int left=pkt.size;
@@ -845,7 +919,7 @@ uint8_t MySQL_Protocol::generate_pkt_row3(MySQL_ResultSet *myrs, unsigned int *l
 			myhdr.pkt_length=0xFFFFFF;
 			memcpy(pkt2.ptr, &myhdr, sizeof(mysql_hdr));
 			// we are writing a large packet (over 16MB), we assume we are always outside the buffer
-			myrs->PSarrayOUT->add(pkt2.ptr,pkt2.size);
+			myrs->PSarrayOUT.add(pkt2.ptr,pkt2.size);
 			copied+=0xFFFFFF;
 			left-=0xFFFFFF;
 		}
@@ -858,7 +932,7 @@ uint8_t MySQL_Protocol::generate_pkt_row3(MySQL_ResultSet *myrs, unsigned int *l
 		myhdr.pkt_length=left-sizeof(mysql_hdr);
 		memcpy(pkt2.ptr, &myhdr, sizeof(mysql_hdr));
 		// we are writing a large packet (over 16MB), we assume we are always outside the buffer
-		myrs->PSarrayOUT->add(pkt2.ptr,pkt2.size);
+		myrs->PSarrayOUT.add(pkt2.ptr,pkt2.size);
 	}
 	if (len) { *len=pkt.size+(pkt_sid-sequence_id)*sizeof(mysql_hdr); }
 	if (pkt.size >= (0xFFFFFF+sizeof(mysql_hdr))) {
@@ -1329,7 +1403,7 @@ bool MySQL_Protocol::process_pkt_handshake_response(unsigned char *pkt, unsigned
 					ret=true;
 				}
 			} else {
-				if (session_type == PROXYSQL_SESSION_MYSQL || session_type == PROXYSQL_SESSION_SQLITE) {
+				if (session_type == PROXYSQL_SESSION_MYSQL || session_type == PROXYSQL_SESSION_SQLITE || PROXYSQL_SESSION_ADMIN || PROXYSQL_SESSION_STATS) {
 					ret=proxy_scramble_sha1((char *)pass,(*myds)->myconn->scramble_buff,password+1, reply);
 					if (ret) {
 						if (sha1_pass==NULL) {
@@ -1638,17 +1712,18 @@ MySQL_ResultSet::MySQL_ResultSet(MySQL_Protocol *_myprot, MYSQL_RES *_res, MYSQL
 	myprot=_myprot;
 	mysql=_my;
 	buffer=NULL;
-	if (_stmt==NULL) { // we allocate this buffer only for not prepared statements
+	//if (_stmt==NULL) { // we allocate this buffer only for not prepared statements
+	// removing the previous assumption. We allocate this buffer also for prepared statements
 		buffer=(unsigned char *)malloc(RESULTSET_BUFLEN);
-	}
+	//}
 	buffer_used=0;
 	myds=NULL;
 	sid=0;
-	PSarrayOUT = NULL;
+	//PSarrayOUT = NULL;
 	if (myprot) { // if myprot = NULL , this is a mirror
 		myds=myprot->get_myds();
 		sid=myds->pkt_sid+1;
-		PSarrayOUT = new PtrSizeArray(8);
+		//PSarrayOUT = new PtrSizeArray(8);
 	}
 	result=_res;
 	resultset_size=0;
@@ -1660,15 +1735,15 @@ MySQL_ResultSet::MySQL_ResultSet(MySQL_Protocol *_myprot, MYSQL_RES *_res, MYSQL
 	if (myprot==NULL) {
 		return; // this is a mirror
 	}
-	myprot->generate_pkt_column_count(false,&pkt.ptr,&pkt.size,sid,num_fields);
+	myprot->generate_pkt_column_count(false,&pkt.ptr,&pkt.size,sid,num_fields,this);
 	sid++;
-	PSarrayOUT->add(pkt.ptr,pkt.size);
+	//PSarrayOUT->add(pkt.ptr,pkt.size);
 	resultset_size+=pkt.size;
 	// columns description
 	for (unsigned int i=0; i<num_fields; i++) {
 		MYSQL_FIELD *field=mysql_fetch_field(result);
-		myprot->generate_pkt_field(false,&pkt.ptr,&pkt.size,sid,field->db,field->table,field->org_table,field->name,field->org_name,field->charsetnr,field->length,field->type,field->flags,field->decimals,false,0,NULL);
-		PSarrayOUT->add(pkt.ptr,pkt.size);
+		myprot->generate_pkt_field(false,&pkt.ptr,&pkt.size,sid,field->db,field->table,field->org_table,field->name,field->org_name,field->charsetnr,field->length,field->type,field->flags,field->decimals,false,0,NULL,this);
+		//PSarrayOUT->add(pkt.ptr,pkt.size);
 		resultset_size+=pkt.size;
 		sid++;
 	}
@@ -1676,11 +1751,23 @@ MySQL_ResultSet::MySQL_ResultSet(MySQL_Protocol *_myprot, MYSQL_RES *_res, MYSQL
 	unsigned int nTrx=myds->sess->NumActiveTransactions();
 	uint16_t setStatus = (nTrx ? SERVER_STATUS_IN_TRANS : 0 );
 	if (myds->sess->autocommit) setStatus += SERVER_STATUS_AUTOCOMMIT;
-	myprot->generate_pkt_EOF(false,&pkt.ptr,&pkt.size,sid,0,mysql->server_status|setStatus);
-	sid++;
-	PSarrayOUT->add(pkt.ptr,pkt.size);
-	resultset_size+=pkt.size;
+	setStatus |= ( mysql->server_status & ~SERVER_STATUS_AUTOCOMMIT ); // get flags from server_status but ignore autocommit
+	setStatus = setStatus & ~SERVER_STATUS_CURSOR_EXISTS; // Do not send cursor #1128
+//	if (_stmt) { // binary protocol , we also assume we have ALL the resultset
+//		myprot->generate_pkt_EOF(false,&pkt.ptr,&pkt.size,sid,0,mysql->server_status|setStatus);
+//		sid++;
+//		PSarrayOUT.add(pkt.ptr,pkt.size);
+//		resultset_size+=pkt.size;
+	//} else {
+		if (RESULTSET_BUFLEN <= (buffer_used + 9)) {
+			buffer_to_PSarrayOut();
+		}
+		myprot->generate_pkt_EOF(false, NULL, NULL, sid, 0, setStatus, this);
+		sid++;
+		resultset_size += 9;
+	//}
 	if (_stmt) { // binary protocol , we also assume we have ALL the resultset
+		buffer_to_PSarrayOut();
 		unsigned long long total_size=0;
 		MYSQL_ROWS *r=_stmt->result.data;
 		if (r) {
@@ -1717,7 +1804,7 @@ MySQL_ResultSet::MySQL_ResultSet(MySQL_Protocol *_myprot, MYSQL_RES *_res, MYSQL
 			}
 			total_size+=sizeof(mysql_hdr);
 		}
-		PSarrayOUT->add(pkt.ptr,pkt.size);
+		PSarrayOUT.add(pkt.ptr,pkt.size);
 		resultset_size+=pkt.size;
 		}
 		add_eof();
@@ -1726,13 +1813,13 @@ MySQL_ResultSet::MySQL_ResultSet(MySQL_Protocol *_myprot, MYSQL_RES *_res, MYSQL
 
 MySQL_ResultSet::~MySQL_ResultSet() {
 	PtrSize_t pkt;
-	if (PSarrayOUT) {
-		while (PSarrayOUT->len) {
-			PSarrayOUT->remove_index_fast(0,&pkt);
+	//if (PSarrayOUT) {
+		while (PSarrayOUT.len) {
+			PSarrayOUT.remove_index_fast(0,&pkt);
 			l_free(pkt.size, pkt.ptr);
 		}
-		delete PSarrayOUT;
-	}
+		//delete PSarrayOUT;
+	//}
 	if (buffer) {
 		free(buffer);
 		buffer=NULL;
@@ -1804,14 +1891,22 @@ unsigned int MySQL_ResultSet::add_row2(MYSQL_ROWS *row, unsigned char *offset) {
 void MySQL_ResultSet::add_eof() {
 	PtrSize_t pkt;
 	if (myprot) {
-		buffer_to_PSarrayOut();
 		unsigned int nTrx=myds->sess->NumActiveTransactions();
 		uint16_t setStatus = (nTrx ? SERVER_STATUS_IN_TRANS : 0 );
 		if (myds->sess->autocommit) setStatus += SERVER_STATUS_AUTOCOMMIT;
-		myprot->generate_pkt_EOF(false,&pkt.ptr,&pkt.size,sid,0,mysql->server_status|setStatus);
-		PSarrayOUT->add(pkt.ptr,pkt.size);
+		setStatus |= ( mysql->server_status & ~SERVER_STATUS_AUTOCOMMIT ); // get flags from server_status but ignore autocommit
+		setStatus = setStatus & ~SERVER_STATUS_CURSOR_EXISTS; // Do not send cursor #1128
+		//myprot->generate_pkt_EOF(false,&pkt.ptr,&pkt.size,sid,0,mysql->server_status|setStatus);
+		//PSarrayOUT->add(pkt.ptr,pkt.size);
+		//sid++;
+		//resultset_size+=pkt.size;
+		if (RESULTSET_BUFLEN <= (buffer_used + 9)) {
+			buffer_to_PSarrayOut();
+		}
+		myprot->generate_pkt_EOF(false, NULL, NULL, sid, 0, setStatus, this);
 		sid++;
-		resultset_size+=pkt.size;
+		resultset_size += 9;
+		buffer_to_PSarrayOut(true);
 	}
 	resultset_completed=true;
 }
@@ -1828,7 +1923,7 @@ void MySQL_ResultSet::add_err(MySQL_Data_Stream *_myds) {
 		} else {
 			myprot->generate_pkt_ERR(false,&pkt.ptr,&pkt.size,sid,mysql_errno(_mysql),sqlstate,mysql_error(_mysql));
 		}
-		PSarrayOUT->add(pkt.ptr,pkt.size);
+		PSarrayOUT.add(pkt.ptr,pkt.size);
 		sid++;
 		resultset_size+=pkt.size;
 	}
@@ -1838,21 +1933,27 @@ void MySQL_ResultSet::add_err(MySQL_Data_Stream *_myds) {
 bool MySQL_ResultSet::get_resultset(PtrSizeArray *PSarrayFinal) {
 	transfer_started=true;
 	if (myprot) {
-		PSarrayFinal->copy_add(PSarrayOUT,0,PSarrayOUT->len);
-		while (PSarrayOUT->len)
-			PSarrayOUT->remove_index(PSarrayOUT->len-1,NULL);
+		PSarrayFinal->copy_add(&PSarrayOUT,0,PSarrayOUT.len);
+		while (PSarrayOUT.len)
+			PSarrayOUT.remove_index(PSarrayOUT.len-1,NULL);
 	}
 	return resultset_completed;
 }
 
-void MySQL_ResultSet::buffer_to_PSarrayOut() {
+void MySQL_ResultSet::buffer_to_PSarrayOut(bool _last) {
 	if (buffer_used==0)
 		return;	// exit immediately if the buffer is empty
 	if (buffer_used < RESULTSET_BUFLEN/2) {
-		buffer=(unsigned char *)realloc(buffer,buffer_used);
+		if (_last == false) {
+			buffer=(unsigned char *)realloc(buffer,buffer_used);
+		}
+	}
+	PSarrayOUT.add(buffer,buffer_used);
+	if (_last) {
+		buffer = NULL;
+	} else {
+		buffer=(unsigned char *)malloc(RESULTSET_BUFLEN);
 	}
-	PSarrayOUT->add(buffer,buffer_used);
-	buffer=(unsigned char *)malloc(RESULTSET_BUFLEN);
 	buffer_used=0;
 }
 
@@ -1860,13 +1961,13 @@ unsigned long long MySQL_ResultSet::current_size() {
 	unsigned long long intsize=0;
 	intsize+=sizeof(MySQL_ResultSet);
 	intsize+=RESULTSET_BUFLEN; // size of buffer
-	if (PSarrayOUT==NULL)	// see bug #699
+	if (PSarrayOUT.len==0)	// see bug #699
 		return intsize;
 	intsize+=sizeof(PtrSizeArray);
-	intsize+=(PSarrayOUT->size*sizeof(PtrSize_t *));
+	intsize+=(PSarrayOUT.size*sizeof(PtrSize_t *));
 	unsigned int i;
-	for (i=0; i<PSarrayOUT->len; i++) {
-		PtrSize_t *pkt=PSarrayOUT->index(i);
+	for (i=0; i<PSarrayOUT.len; i++) {
+		PtrSize_t *pkt=PSarrayOUT.index(i);
 		if (pkt->size>RESULTSET_BUFLEN) {
 			intsize+=pkt->size;
 		} else {
