@@ -176,7 +176,7 @@ class QP_query_digest_stats {
 
 		time_t __now;
     time(&__now);
-		
+
 		unsigned long long curtime=monotonic_time();
 
 		time_t seen_time;
@@ -435,7 +435,8 @@ QP_rule_t * Query_Processor::new_query_rule(int rule_id, bool active, char *user
 	newQR->negate_match_pattern=negate_match_pattern;
 	newQR->re_modifiers=0;
 	{
-		tokenizer_t tok = tokenizer( re_modifiers, ",", TOKENIZER_NO_EMPTIES );
+		tokenizer_t tok;
+		tokenizer( &tok, re_modifiers, ",", TOKENIZER_NO_EMPTIES );
 		const char* token;
 		for (token = tokenize( &tok ); token; token = tokenize( &tok )) {
 			if (strncasecmp(token,(char *)"CASELESS",strlen((char *)"CASELESS"))==0) {
@@ -576,7 +577,7 @@ SQLite3_result * Query_Processor::get_stats_commands_counters() {
 	result->add_column_definition(SQLITE_TEXT,"cnt_5s");
 	result->add_column_definition(SQLITE_TEXT,"cnt_10s");
 	result->add_column_definition(SQLITE_TEXT,"cnt_INFs");
-	for (int i=0;i<MYSQL_COM_QUERY___NONE;i++) {
+	for (int i=0 ; i < MYSQL_COM_QUERY__UNINITIALIZED ; i++) {
 		char **pta=commands_counters[i]->get_row();
 		result->add_row(pta);
 		commands_counters[i]->free_row(pta);
@@ -974,12 +975,12 @@ __internal_loop:
 			//sess->query_info.flagOUT=flagIN;
     }
     if (qr->reconnect >= 0) {
-			// Note: negative reconnect means this rule doesn't change 
+			// Note: negative reconnect means this rule doesn't change
       proxy_debug(PROXY_DEBUG_MYSQL_QUERY_PROCESSOR, 5, "query rule %d has set reconnect: %d. Query will%s be rexecuted if connection is lost\n", qr->rule_id, qr->reconnect, (qr->reconnect == 0 ? " NOT" : "" ));
       ret->reconnect=qr->reconnect;
     }
     if (qr->timeout >= 0) {
-			// Note: negative timeout means this rule doesn't change 
+			// Note: negative timeout means this rule doesn't change
       proxy_debug(PROXY_DEBUG_MYSQL_QUERY_PROCESSOR, 5, "query rule %d has set timeout: %d. Query will%s be interrupted if exceeding %dms\n", qr->rule_id, qr->timeout, (qr->timeout == 0 ? " NOT" : "" ) , qr->timeout);
       ret->timeout=qr->timeout;
     }
@@ -989,7 +990,7 @@ __internal_loop:
       ret->retries=qr->retries;
     }
     if (qr->delay >= 0) {
-			// Note: negative delay means this rule doesn't change 
+			// Note: negative delay means this rule doesn't change
       proxy_debug(PROXY_DEBUG_MYSQL_QUERY_PROCESSOR, 5, "query rule %d has set delay: %d. Session will%s be paused for %dms\n", qr->rule_id, qr->delay, (qr->delay == 0 ? " NOT" : "" ) , qr->delay);
       ret->delay=qr->delay;
     }
@@ -1019,7 +1020,7 @@ __internal_loop:
       ret->OK_msg=strdup(qr->OK_msg);
     }
     if (qr->cache_ttl >= 0) {
-			// Note: negative TTL means this rule doesn't change 
+			// Note: negative TTL means this rule doesn't change
       proxy_debug(PROXY_DEBUG_MYSQL_QUERY_PROCESSOR, 5, "query rule %d has set cache_ttl: %d. Query will%s hit the cache\n", qr->rule_id, qr->cache_ttl, (qr->cache_ttl == 0 ? " NOT" : "" ));
       ret->cache_ttl=qr->cache_ttl;
     }
@@ -1039,7 +1040,7 @@ __internal_loop:
       ret->log=qr->log;
     }
     if (qr->destination_hostgroup >= 0) {
-			// Note: negative hostgroup means this rule doesn't change 
+			// Note: negative hostgroup means this rule doesn't change
       proxy_debug(PROXY_DEBUG_MYSQL_QUERY_PROCESSOR, 5, "query rule %d has set destination hostgroup: %d\n", qr->rule_id, qr->destination_hostgroup);
       ret->destination_hostgroup=qr->destination_hostgroup;
     }
@@ -1076,7 +1077,7 @@ __internal_loop:
 			}
 		}
 	}
-	
+
 __exit_process_mysql_query:
 	// FIXME : there is too much data being copied around
 	if (len < stackbuffer_size) {
@@ -1231,7 +1232,7 @@ void Query_Processor::update_query_digest(SQP_par_t *qp, int hid, MySQL_Connecti
 #else
 	spin_wrlock(&digest_rwlock);
 #endif
-	QP_query_digest_stats *qds;	
+	QP_query_digest_stats *qds;
 
 	std::unordered_map<uint64_t, void *>::iterator it;
 	it=digest_umap.find(qp->digest_total);
@@ -1277,13 +1278,23 @@ enum MYSQL_COM_QUERY_command Query_Processor::__query_parser_command_type(SQP_pa
 	enum MYSQL_COM_QUERY_command ret=MYSQL_COM_QUERY_UNKNOWN;
 	char c1;
 
-  tokenizer_t tok = tokenizer( text, " ", TOKENIZER_NO_EMPTIES );
-  char* token=NULL;
+	tokenizer_t tok;
+	tokenizer( &tok, text, " ", TOKENIZER_NO_EMPTIES );
+	char* token=NULL;
+__get_token:
 	token=(char *)tokenize(&tok);
 	if (token==NULL) {
 		goto __exit__query_parser_command_type;
 	}
-
+__remove_paranthesis:
+	if (token[0] == '(') {
+		if (strlen(token) > 1) {
+			token++;
+			goto __remove_paranthesis;
+		} else {
+			goto __get_token;
+		}
+	}
 	c1=token[0];
 	proxy_debug(PROXY_DEBUG_MYSQL_COM, 5, "Command:%s Prefix:%c\n", token, c1);
 	switch (c1) {
@@ -1622,7 +1633,8 @@ __exit__query_parser_command_type:
 
 bool Query_Processor::query_parser_first_comment(Query_Processor_Output *qpo, char *fc) {
 	bool ret=false;
-	tokenizer_t tok = tokenizer( fc, ";", TOKENIZER_NO_EMPTIES );
+	tokenizer_t tok;
+	tokenizer( &tok, fc, ";", TOKENIZER_NO_EMPTIES );
 	const char* token;
 	for ( token = tokenize( &tok ) ; token ;  token = tokenize( &tok ) ) {
 		char *key=NULL;
@@ -1669,9 +1681,10 @@ bool Query_Processor::query_parser_first_comment(Query_Processor_Output *qpo, ch
 				}
 			}
 		}
+
+		proxy_debug(PROXY_DEBUG_MYSQL_QUERY_PROCESSOR, 5, "Variables in comment %s , key=%s , value=%s\n", token, key, value);
 		free(key);
 		free(value);
-		proxy_debug(PROXY_DEBUG_MYSQL_QUERY_PROCESSOR, 5, "Variables in comment %s , key=%s , value=%s\n", token, key, value);
 	}
 	free_tokenizer( &tok );
 	return ret;
