@@ -23,6 +23,16 @@ mysql_status(short event, short cont) {
 	return status;
 }
 
+// Defining list of session variables for comparison with query digest to disable multiplexing for "SET <variable_name>" commands
+static char * session_vars[]= {
+	// For issue #555 , multiplexing is disabled if --safe-updates is used
+	(char *)"SQL_SAFE_UPDATES=?,SQL_SELECT_LIMIT=?,MAX_JOIN_SIZE=?",
+	(char *)"FOREIGN_KEY_CHECKS",
+	(char *)"UNIQUE_CHECKS",
+	(char *)"AUTO_INCREMENT_INCREMENT",
+	(char *)"AUTO_INCREMENT_OFFSET",
+	(char *)"GROUP_CONCAT_MAX_LEN"
+};
 
 MySQL_Connection_userinfo::MySQL_Connection_userinfo() {
 	username=NULL;
@@ -196,6 +206,9 @@ MySQL_Connection::~MySQL_Connection() {
 		delete userinfo;
 		userinfo=NULL;
 	}
+	if (local_stmts) {
+		delete local_stmts;
+	}
 	if (mysql) {
 		// always decrease the counter
 		if (ret_mysql)
@@ -207,9 +220,6 @@ MySQL_Connection::~MySQL_Connection() {
 	if (MyRS) {
 		delete MyRS;
 	}
-	if (local_stmts) {
-		delete local_stmts;
-	}
 	if (query.stmt) {
 		query.stmt=NULL;
 	}
@@ -471,7 +481,7 @@ void MySQL_Connection::set_names_start() {
 		proxy_error("Not existing charset number %u\n", options.charset);
 		assert(0);
 	}
-	async_exit_status = mysql_set_character_set_start(&interr,mysql, c->csname);
+	async_exit_status = mysql_set_character_set_start(&interr,mysql, NULL, options.charset);
 }
 
 void MySQL_Connection::set_names_cont(short event) {
@@ -561,7 +571,7 @@ void MySQL_Connection::set_is_client() {
 #define NEXT_IMMEDIATE(new_st) do { async_state_machine = new_st; goto handler_again; } while (0)
 
 MDB_ASYNC_ST MySQL_Connection::handler(short event) {
-	unsigned int processed_bytes=0;	// issue #527 : this variable will store the amount of bytes processed during this event
+	unsigned long long processed_bytes=0;	// issue #527 : this variable will store the amount of bytes processed during this event
 	if (mysql==NULL) {
 		// it is the first time handler() is being called
 		async_state_machine=ASYNC_CONNECT_START;
@@ -907,6 +917,15 @@ handler_again:
 			}
 			break;
 		case ASYNC_USE_RESULT_CONT:
+			{
+				unsigned int buffered_data=0;
+				buffered_data = myds->sess->client_myds->PSarrayOUT->len * RESULTSET_BUFLEN;
+				buffered_data += myds->sess->client_myds->resultset->len * RESULTSET_BUFLEN;
+				if (buffered_data > (unsigned int)mysql_thread___threshold_resultset_size*8) {
+					next_event(ASYNC_USE_RESULT_CONT); // we temporarily pause . See #1232
+					break;
+				}
+			}
 			if (async_fetch_row_start==false) {
 				async_exit_status=mysql_fetch_row_start(&mysql_row,mysql_result);
 				async_fetch_row_start=true;
@@ -922,7 +941,11 @@ handler_again:
 					__sync_fetch_and_add(&parent->bytes_recv,br);
 					myds->sess->thread->status_variables.queries_backends_bytes_recv+=br;
 					processed_bytes+=br;	// issue #527 : this variable will store the amount of bytes processed during this event
-					if (processed_bytes > (unsigned int)mysql_thread___threshold_resultset_size*8) {
+					if (
+						(processed_bytes > (unsigned int)mysql_thread___threshold_resultset_size*8)
+							||
+						( mysql_thread___throttle_ratio_server_to_client && (processed_bytes > (unsigned long long)mysql_thread___throttle_max_bytes_per_second_to_client/10*(unsigned long long)mysql_thread___throttle_ratio_server_to_client) )
+					) {
 						next_event(ASYNC_USE_RESULT_CONT); // we temporarily pause
 					} else {
 						NEXT_IMMEDIATE(ASYNC_USE_RESULT_CONT); // we continue looping
@@ -1421,15 +1444,15 @@ void MySQL_Connection::async_free_result() {
 		query.ptr=NULL;
 		query.length=0;
 	}
+	if (query.stmt_result) {
+		mysql_free_result(query.stmt_result);
+		query.stmt_result=NULL;
+	}
 	if (userinfo) {
 		// if userinfo is NULL , the connection is being destroyed
 		// because it is reset on destructor ( ~MySQL_Connection() )
 		// therefore this section is skipped completely
 		// this should prevent bug #1046
-		if (query.stmt_result) {
-			mysql_free_result(query.stmt_result);
-			query.stmt_result=NULL;
-		}
 		if (query.stmt) {
 			if (query.stmt->mysql) {
 				if (query.stmt->mysql == mysql) { // extra check
@@ -1450,7 +1473,7 @@ void MySQL_Connection::async_free_result() {
 	}
 }
 
-/*
+
 bool MySQL_Connection::IsActiveTransaction() {
 	bool ret=false;
 	if (mysql) {
@@ -1458,10 +1481,16 @@ bool MySQL_Connection::IsActiveTransaction() {
 		if (ret == false && (mysql)->net.last_errno) {
 			ret = true;
 		}
+		if (ret == false) {
+			bool r = ( mysql_thread___autocommit_false_is_transaction || mysql_thread___forward_autocommit );
+			if ( r && (IsAutoCommit() == false) ) {
+				ret = true;
+			}
+		}
 	}
 	return ret;
 }
-*/
+
 
 bool MySQL_Connection::IsAutoCommit() {
 	bool ret=false;
@@ -1519,13 +1548,12 @@ void MySQL_Connection::ProcessQueryAndSetStatusFlags(char *query_digest_text) {
 			}
 		}
 		if (strncasecmp(query_digest_text,"SET ",4)==0) {
-			// For issue #555 , multiplexing is disabled if --safe-updates is used
-			if (strcasecmp(query_digest_text,"SET SQL_SAFE_UPDATES=?,SQL_SELECT_LIMIT=?,MAX_JOIN_SIZE=?")==0) {
-				set_status_user_variable(true);
-			} else if (strcasecmp(query_digest_text,"SET FOREIGN_KEY_CHECKS=?")==0) { // see #835
-				set_status_user_variable(true);
-			} else if (strcasecmp(query_digest_text,"SET UNIQUE_CHECKS=?")==0) { // see #835
-				set_status_user_variable(true);
+			// For issue #555 , multiplexing is disabled if --safe-updates is used (see session_vars definition)
+			for (unsigned int i = 0; i < sizeof(session_vars); i++) {
+				if (strcasestr(query_digest_text,session_vars[i])!=NULL)  {
+					set_status_user_variable(true);
+					break;
+				}
 			}
 		}
 	}
