@@ -103,6 +103,7 @@ MySQL_Data_Stream::MySQL_Data_Stream() {
 	connect_retries_on_failure=0;
 	max_connect_time=0;
 	wait_until=0;
+	pause_until=0;
 	connect_tries=0;
 	poll_fds_idx=-1;
 	resultset_length=0;
@@ -328,7 +329,7 @@ int MySQL_Data_Stream::write_to_net() {
 	VALGRIND_ENABLE_ERROR_REPORTING;
 	if (bytes_io < 0) {
 		if (encrypted==false)	{
-			if (mypolls->fds[poll_fds_idx].revents & POLLOUT) { // in write_to_net_poll() we has remove this safety
+			if ((poll_fds_idx < 0) || (mypolls->fds[poll_fds_idx].revents & POLLOUT)) { // in write_to_net_poll() we has remove this safety
                                                           // so we enforce it here
 				shut_soft();
 			}
@@ -352,6 +353,12 @@ bool MySQL_Data_Stream::available_data_out() {
 	return false;
 }
 
+void MySQL_Data_Stream::remove_pollout() {
+	struct pollfd *_pollfd;
+	_pollfd=&mypolls->fds[poll_fds_idx];
+	_pollfd->events = 0;
+}
+
 void MySQL_Data_Stream::set_pollout() {
 	struct pollfd *_pollfd;
 	_pollfd=&mypolls->fds[poll_fds_idx];
