@@ -282,3 +282,103 @@ char *SQLite3_result::checksum() {
 	sprintf(buf,"0x%X%X", d32[0], d32[1]);
 	return strdup(buf);
 }
+
+void SQLite3_result::dump_to_stderr() {
+	if (columns == 0) return;
+	size_t *columns_lengths = (size_t *)malloc(sizeof(size_t)*columns);
+
+	int i = 0;
+	for (i = 0; i<columns; i++) {
+		columns_lengths[i] = 0;
+	}
+	i = 0;
+	for (std::vector<SQLite3_column *>::iterator it=column_definition.begin() ; it!=column_definition.end(); ++it) {
+		SQLite3_column *r=*it;
+		size_t len = strlen(r->name);
+		if (len > columns_lengths[i]) {
+			columns_lengths[i] = len;
+			i++;
+		}
+	}
+	for (std::vector<SQLite3_row *>::iterator it=rows.begin() ; it!=rows.end(); ++it) {
+		SQLite3_row *r=*it;
+		for (int i=0; i<columns;i++) {
+			if (r->fields[i]) {
+				if ((unsigned int)r->sizes[i] > columns_lengths[i]) {
+					columns_lengths[i] = r->sizes[i];
+				}
+			} else {
+				if (columns_lengths[i] < 4) {
+					columns_lengths[i] = 4; // NULL
+				}
+			}
+		}
+	}
+	string s;
+	s = "+";
+	for (i=0; i<columns; i++) {
+		unsigned int j;
+		for (j=0; j < columns_lengths[i] + 2; j++) {
+			s.append("-");
+		}
+		s.append("+");
+	}
+	fprintf(stderr,"%s\n",s.c_str());
+	s = "|";
+	i = 0;
+	for (std::vector<SQLite3_column *>::iterator it=column_definition.begin() ; it!=column_definition.end(); ++it) {
+		SQLite3_column *r=*it;
+		size_t len = strlen(r->name);
+		s.append(" ");
+		s.append(r->name);
+		unsigned int j;
+		for (j=0; j < columns_lengths[i] - len + 1 ; j++) {
+			s.append(" ");
+		}
+		s.append("|");
+		i++;
+	}
+	fprintf(stderr,"%s\n",s.c_str());
+	s = "+";
+	for (i=0; i<columns; i++) {
+		unsigned int j;
+		for (j=0; j < columns_lengths[i] + 2 ; j++) {
+			s.append("-");
+		}
+		s.append("+");
+	}
+	fprintf(stderr,"%s\n",s.c_str());
+
+	for (std::vector<SQLite3_row *>::iterator it=rows.begin() ; it!=rows.end(); ++it) {
+		SQLite3_row *r=*it;
+		s = "|";
+		i = 0;
+		for (int i=0; i<columns;i++) {
+			s.append(" ");
+			int len = 0;
+			if (r->fields[i]) {
+				len = r->sizes[i];
+				s.append(r->fields[i]);
+			} else {
+				len = 4;
+				s.append("NULL");
+			}
+			unsigned int j;
+			for (j=0; j < columns_lengths[i] - len + 1 ; j++) {
+				s.append(" ");
+			}
+			s.append("|");
+		}
+		fprintf(stderr,"%s\n",s.c_str());
+	}
+
+	s = "+";
+	for (i=0; i<columns; i++) {
+		unsigned int j;
+		for (j=0; j < columns_lengths[i] + 2 ; j++) {
+			s.append("-");
+		}
+		s.append("+");
+	}
+	fprintf(stderr,"%s\n",s.c_str());
+}
