@@ -39,18 +39,20 @@ Thread::~Thread()
     }
 }
 
-int Thread::start(bool jemalloc_tcache)
+int Thread::start(unsigned int ss, bool jemalloc_tcache)
 {
     pthread_attr_t attr;
     pthread_attr_init(&attr);
-    pthread_attr_setstacksize (&attr, 64*1024);
+    pthread_attr_setstacksize (&attr, ss*1024);
     int result = pthread_create(&m_tid, &attr, runThread, this);
     if (result == 0) {
         m_running = 1;
     }
 		if (jemalloc_tcache==false) {
+#ifndef NOJEM
 			bool cache=false;
 			mallctl("thread.tcache.enabled", NULL, NULL, &cache, sizeof(bool));
+#endif /* NOJEM */
 		}
     return result;
 }
