@@ -2,7 +2,7 @@
 #include <thread>
 #include "btree_map.h"
 #include "proxysql.h"
-#ifdef __FreeBSD__
+#if defined(__FreeBSD__) || defined(__APPLE__)
 #include <fcntl.h>
 #endif
 
@@ -22,6 +22,24 @@
 #endif
 
 
+
+// this fuction will be called as a deatached thread
+static void * waitpid_thread(void *arg) {
+	pid_t *cpid_ptr=(pid_t *)arg;
+	int status;
+	waitpid(*cpid_ptr, &status, 0);
+	free(cpid_ptr);
+	return NULL;
+}
+
+
+
+// Note: if you are running ProxySQL under gdb, you may consider setting this
+// variable immediately to 1
+// Example:
+// set disable_watchdog=1
+volatile int disable_watchdog = 0;
+
 void parent_open_error_log() {
 	if (GloVars.global.foreground==false) {
 		int outfd=0;
@@ -202,6 +220,9 @@ ClickHouse_Server *GloClickHouseServer;
 
 ProxySQL_Cluster *GloProxyCluster = NULL;
 
+ProxySQL_Statistics *GloProxyStats = NULL;
+
+
 void * mysql_worker_thread_func(void *arg) {
 
 //	__thr_sfp=l_mem_init();
@@ -273,9 +294,29 @@ void * mysql_shared_query_cache_funct(void *arg) {
 void ProxySQL_Main_process_global_variables(int argc, const char **argv) {
 	GloVars.parse(argc,argv);
 	GloVars.process_opts_pre();
+	GloVars.restart_on_missing_heartbeats = 10; // default
 	// alwasy try to open a config file
 	if (GloVars.confFile->OpenFile(GloVars.config_file) == true) {
 		GloVars.configfile_open=true;
+		const Setting& root = GloVars.confFile->cfg->getRoot();
+		if (root.exists("restart_on_missing_heartbeats")==true) {
+			// restart_on_missing_heartbeats datadir from config file
+			int restart_on_missing_heartbeats;
+			bool rc;
+			rc=root.lookupValue("restart_on_missing_heartbeats", restart_on_missing_heartbeats);
+			if (rc==true) {
+				GloVars.restart_on_missing_heartbeats=restart_on_missing_heartbeats;
+			}
+		}
+		if (root.exists("execute_on_exit_failure")==true) {
+			// restart_on_missing_heartbeats datadir from config file
+			string execute_on_exit_failure;
+			bool rc;
+			rc=root.lookupValue("execute_on_exit_failure", execute_on_exit_failure);
+			if (rc==true) {
+				GloVars.execute_on_exit_failure=strdup(execute_on_exit_failure.c_str());
+			}
+		}
 	} else {
 		proxy_warning("Unable to open config file %s\n", GloVars.config_file); // issue #705
 	}
@@ -283,7 +324,7 @@ void ProxySQL_Main_process_global_variables(int argc, const char **argv) {
 	if (GloVars.__cmd_proxysql_datadir==NULL) {
 		// datadir was not specified , try to read config file
 		if (GloVars.configfile_open==true) {
-			const Setting& root = GloVars.confFile->cfg->getRoot(); 
+			const Setting& root = GloVars.confFile->cfg->getRoot();
 			if (root.exists("datadir")==true) {
 				// reading datadir from config file
 				std::string datadir;
@@ -298,6 +339,20 @@ void ProxySQL_Main_process_global_variables(int argc, const char **argv) {
 				// datadir was not specified in config file
 				GloVars.datadir=strdup(t);
 			}
+			if (root.exists("restart_on_missing_heartbeats")==true) {
+				// restart_on_missing_heartbeats datadir from config file
+				int restart_on_missing_heartbeats;
+				bool rc;
+				rc=root.lookupValue("restart_on_missing_heartbeats", restart_on_missing_heartbeats);
+				if (rc==true) {
+					GloVars.restart_on_missing_heartbeats=restart_on_missing_heartbeats;
+				} else {
+					GloVars.restart_on_missing_heartbeats = 10; // default
+				}
+			} else {
+				// restart_on_missing_heartbeats was not specified in config file
+				GloVars.restart_on_missing_heartbeats = 10; // default
+			}
 		} else {
 			// config file not readable
 			GloVars.datadir=strdup(t);
@@ -311,6 +366,9 @@ void ProxySQL_Main_process_global_variables(int argc, const char **argv) {
 	GloVars.admindb=(char *)malloc(strlen(GloVars.datadir)+strlen((char *)"proxysql.db")+2);
 	sprintf(GloVars.admindb,"%s/%s",GloVars.datadir, (char *)"proxysql.db");
 
+	GloVars.statsdb_disk=(char *)malloc(strlen(GloVars.datadir)+strlen((char *)"proxysql_stats.db")+2);
+	sprintf(GloVars.statsdb_disk,"%s/%s",GloVars.datadir, (char *)"proxysql_stats.db");
+
 	GloVars.errorlog=(char *)malloc(strlen(GloVars.datadir)+strlen((char *)"proxysql.log")+2);
 	sprintf(GloVars.errorlog,"%s/%s",GloVars.datadir, (char *)"proxysql.log");
 
@@ -355,6 +413,9 @@ void ProxySQL_Main_init_Admin_module() {
 	GloProxyCluster = new ProxySQL_Cluster();
 	GloProxyCluster->init();
 	GloProxyCluster->print_version();
+	GloProxyStats = new ProxySQL_Statistics();
+	//GloProxyStats->init();
+	GloProxyStats->print_version();
 	GloAdmin = new ProxySQL_Admin();
 	GloAdmin->init();
 	GloAdmin->print_version();
@@ -779,6 +840,43 @@ bool ProxySQL_daemonize_phase2() {
 	return true;
 }
 
+
+void call_execute_on_exit_failure() {
+	if (GloVars.execute_on_exit_failure == NULL) {
+		return;
+	}
+	proxy_error("Trying to call external script after exit failure: %s\n", GloVars.execute_on_exit_failure);
+	pid_t cpid;
+	cpid = fork();
+	if (cpid == -1) {
+		exit(EXIT_FAILURE);
+	}
+	if (cpid == 0) {
+		int rc;
+		rc = system(GloVars.execute_on_exit_failure);
+		if (rc) {
+			proxy_error("Execute on EXIT_FAILURE: Failed to run %s\n", GloVars.execute_on_exit_failure);
+			perror("system()");
+			exit(EXIT_FAILURE);
+		} else {
+			exit(EXIT_SUCCESS);
+		}
+	} else {
+		pthread_attr_t attr;
+		pthread_attr_init(&attr);
+		pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
+		pthread_attr_setstacksize (&attr, 64*1024);
+		pid_t *cpid_ptr=(pid_t *)malloc(sizeof(pid_t));
+		*cpid_ptr=cpid;
+		pthread_t thr;
+		if (pthread_create(&thr, &attr, waitpid_thread, (void *)cpid_ptr) !=0 ) {
+			perror("Thread creation");
+			exit(EXIT_FAILURE);
+		}
+	}
+}
+
+
 bool ProxySQL_daemonize_phase3() {
 	int rc;
 	int status;
@@ -805,6 +903,7 @@ bool ProxySQL_daemonize_phase3() {
 			//daemon_log(LOG_INFO, "ProxySQL exited with code %d . Restarting!\n", rc);
 			parent_open_error_log();
 			proxy_error("ProxySQL exited with code %d . Restarting!\n", rc);
+			call_execute_on_exit_failure();
 			parent_close_error_log();
 			return false;
 		}
@@ -812,6 +911,7 @@ bool ProxySQL_daemonize_phase3() {
 		//daemon_log(LOG_INFO, "ProxySQL crashed. Restarting!\n");
 		parent_open_error_log();
 		proxy_error("ProxySQL crashed. Restarting!\n");
+		call_execute_on_exit_failure();
 		parent_close_error_log();
 		return false;
 	}
@@ -936,8 +1036,74 @@ __start_label:
 #endif
 	}
 
-	while (glovars.shutdown==0) {
-		usleep(500000);   // FIXME: TERRIBLE UGLY
+	{
+		unsigned int missed_heartbeats = 0;
+		unsigned long long previous_time = monotonic_time();
+		unsigned int inner_loops = 0;
+		while (glovars.shutdown==0) {
+			usleep(200000);
+			if (disable_watchdog) {
+				continue;
+			}
+			unsigned long long curtime = monotonic_time();
+			inner_loops++;
+			if (curtime >= inner_loops*300000 + previous_time ) {
+				// if this happens, it means that this very simple loop is blocked
+				// probably we are running under gdb
+				previous_time = curtime;
+				inner_loops = 0;
+				continue;
+			}
+			if (GloMTH) {
+				unsigned long long atomic_curtime = 0;
+				unsigned long long poll_timeout = (unsigned int)GloMTH->variables.poll_timeout;
+				unsigned int threads_missing_heartbeat = 0;
+				poll_timeout += 1000; // add 1 second (rounding up)
+				poll_timeout *= 1000; // convert to us
+				if (curtime < previous_time + poll_timeout) {
+					continue;
+				}
+				previous_time = curtime;
+				inner_loops = 0;
+				unsigned int i;
+				if (GloMTH->mysql_threads) {
+					for (i=0; i<GloMTH->num_threads; i++) {
+						if (GloMTH->mysql_threads[i].worker) {
+							atomic_curtime = GloMTH->mysql_threads[i].worker->atomic_curtime;
+							if (curtime > atomic_curtime + poll_timeout) {
+								threads_missing_heartbeat++;
+							}
+						}
+					}
+				}
+#ifdef IDLE_THREADS
+				if (GloVars.global.idle_threads) {
+					if (GloMTH->mysql_threads) {
+						for (i=0; i<GloMTH->num_threads; i++) {
+							if (GloMTH->mysql_threads_idles[i].worker) {
+								atomic_curtime = GloMTH->mysql_threads_idles[i].worker->atomic_curtime;
+								if (curtime > atomic_curtime + poll_timeout) {
+									threads_missing_heartbeat++;
+								}
+							}
+						}
+					}
+				}
+#endif
+				if (threads_missing_heartbeat) {
+					proxy_error("Watchdog: %u threads missed a heartbeat\n", threads_missing_heartbeat);
+					missed_heartbeats++;
+					if (missed_heartbeats >= (unsigned int)GloVars.restart_on_missing_heartbeats) {
+						if (GloVars.restart_on_missing_heartbeats) {
+							proxy_error("Watchdog: reached %u missed heartbeats. Aborting!\n", missed_heartbeats);
+							assert(0);
+						}
+					}
+				} else {
+					missed_heartbeats = 0;
+				}
+			}
+		}
 	}
 
 __shutdown:
